@@ -19,7 +19,7 @@
 /* This file contains the XDMCP chooser glue */
 
 #include <config.h> 
-#include <gnome.h>
+#include <libgnome/libgnome.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <syslog.h>
@@ -43,8 +43,6 @@
 #include "choose.h"
 #include "xdmcp.h"
 
-static const gchar RCSid[]="$Id: choose.c,v 1.17 2001/10/28 19:13:42 jirka Exp $";
-
 static gint ipending = 0;
 static GSList *indirect = NULL;
 
@@ -101,7 +99,7 @@ gdm_choose_data (const char *data)
 	}
 
 	p = strtok (NULL, " ");
-	if (p == NULL || inet_aton (p, &addr) == 0) {
+	if (p == NULL || inet_pton (AF_INET, p, &addr) == 0) {
 		g_free (msg);
 		return FALSE;
 	}
