@@ -54,14 +54,37 @@ static int GdmMinimalUID = 100;
 
 static GladeXML *xml;
 
+static GList *timeout_widgets = NULL;
+
 static void
 setup_cursor (GdkCursorType type)
 {
 	GdkCursor *cursor = gdk_cursor_new (type);
-	gdk_window_set_cursor (GDK_ROOT_PARENT (), cursor);
-	gdk_cursor_destroy (cursor);
+	gdk_window_set_cursor (gdk_get_default_root_window (), cursor);
+	gdk_cursor_unref (cursor);
 }
 
+static void
+setup_window_cursor (GdkCursorType type)
+{
+	GdkCursor *cursor = gdk_cursor_new (type);
+	GtkWidget *setup_dialog = glade_helper_get
+		(xml, "setup_dialog", GTK_TYPE_WINDOW);
+	if (setup_dialog->window)
+		gdk_window_set_cursor (setup_dialog->window, cursor);
+	gdk_cursor_unref (cursor);
+}
+
+static void
+unsetup_window_cursor (void)
+{
+	GtkWidget *setup_dialog = glade_helper_get
+		(xml, "setup_dialog", GTK_TYPE_WINDOW);
+	if (setup_dialog->window)
+		gdk_window_set_cursor (setup_dialog->window, NULL);
+}
+
+
 static void
 update_greeters (void)
 {
@@ -70,6 +93,9 @@ update_greeters (void)
 	static gboolean shown_error = FALSE;
 	gboolean have_error = FALSE;
 
+	/* recheck for gdm */
+	gdm_running = gdmcomm_check (FALSE /* gui_bitching */);
+
 	if ( ! gdm_running)
 		return;
 
@@ -127,6 +153,24 @@ check_update_error:
 	}
 }
 
+static gboolean
+the_timeout (gpointer data)
+{
+	GtkWidget *widget = data;
+	gboolean (*func) (GtkWidget *);
+
+	func = g_object_get_data (G_OBJECT (widget), "timeout_func");
+
+	if ( ! (*func) (widget)) {
+		g_object_set_data (G_OBJECT (widget), "change_timeout", NULL);
+		g_object_set_data (G_OBJECT (widget), "timeout_func", NULL);
+		timeout_widgets = g_list_remove (timeout_widgets, widget);
+		return FALSE;
+	} else {
+		return TRUE;
+	}
+}
+
 static void
 run_timeout (GtkWidget *widget, guint tm, gboolean (*func) (GtkWidget *))
 {
@@ -134,9 +178,11 @@ run_timeout (GtkWidget *widget, guint tm, gboolean (*func) (GtkWidget *))
 							"change_timeout"));
 	if (id != 0) {
 		g_source_remove (id);
+	} else {
+		timeout_widgets = g_list_prepend (timeout_widgets, widget);
 	}
 
-	id = g_timeout_add (tm, (GSourceFunc)func, widget);
+	id = g_timeout_add (tm, the_timeout, widget);
 	g_object_set_data (G_OBJECT (widget), "timeout_func", func);
 
 	g_object_set_data (G_OBJECT (widget), "change_timeout",
@@ -146,7 +192,13 @@ run_timeout (GtkWidget *widget, guint tm, gboolean (*func) (GtkWidget *))
 static void
 update_key (const char *notify_key)
 {
-	if (notify_key != NULL && gdm_running) {
+	if (notify_key == NULL)
+	       return;
+
+	/* recheck for gdm */
+	gdm_running = gdmcomm_check (FALSE /* gui_bitching */);
+
+	if (gdm_running) {
 		char *ret;
 		char *s = g_strdup_printf ("%s %s", GDM_SUP_UPDATE_CONFIG,
 					   notify_key);
@@ -165,21 +217,18 @@ toggle_timeout (GtkWidget *toggle)
 	const char *key = g_object_get_data (G_OBJECT (toggle), "key");
 	const char *notify_key = g_object_get_data (G_OBJECT (toggle),
 						    "notify_key");
+	VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
 	gboolean val;
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-
-	val = gnome_config_get_bool (key);
+	val = ve_config_get_bool (config, key);
 	if ( ! ve_bool_equal (val, GTK_TOGGLE_BUTTON (toggle)->active)) {
-		gnome_config_set_bool (key, GTK_TOGGLE_BUTTON (toggle)->active);
-
-		gnome_config_sync ();
+		ve_config_set_bool (config, key,
+				    GTK_TOGGLE_BUTTON (toggle)->active);
+		ve_config_save (config, FALSE /*force */);
 
 		update_key (notify_key);
 	}
 
-	gnome_config_pop_prefix ();
-
 	return FALSE;
 }
 
@@ -189,25 +238,22 @@ entry_timeout (GtkWidget *entry)
 	const char *key = g_object_get_data (G_OBJECT (entry), "key");
 	const char *text;
 	char *val;
+	VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
 
 	text = gtk_entry_get_text (GTK_ENTRY (entry));
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-
-	val = gnome_config_get_string (key);
+	val = ve_config_get_string (config, key);
 
 	if (strcmp (ve_sure_string (val), ve_sure_string (text)) != 0) {
-		gnome_config_set_string (key, ve_sure_string (text));
+		ve_config_set_string (config, key, ve_sure_string (text));
 
-		gnome_config_sync ();
+		ve_config_save (config, FALSE /* force */);
 
 		update_key (key);
 	}
 
 	g_free (val);
 
-	gnome_config_pop_prefix ();
-
 	return FALSE;
 }
 
@@ -218,23 +264,20 @@ intspin_timeout (GtkWidget *spin)
 	const char *notify_key = g_object_get_data (G_OBJECT (spin),
 						    "notify_key");
 	int val, new_val;
+	VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
 
 	new_val = gtk_spin_button_get_value_as_int (GTK_SPIN_BUTTON (spin));
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-
-	val = gnome_config_get_int (key);
+	val = ve_config_get_int (config, key);
 
 	if (val != new_val) {
-		gnome_config_set_int (key, new_val);
+		ve_config_set_int (config, key, new_val);
 
-		gnome_config_sync ();
+		ve_config_save (config, FALSE /* force */);
 
 		update_key (notify_key);
 	}
 
-	gnome_config_pop_prefix ();
-
 	return FALSE;
 }
 
@@ -262,25 +305,22 @@ option_timeout (GtkWidget *option_menu)
 	const char *key = g_object_get_data (G_OBJECT (option_menu), "key");
 	const char *new_val;
 	char *val;
+	VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
 
 	new_val = get_str_from_option (GTK_OPTION_MENU (option_menu));
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-
-	val = gnome_config_get_string (key);
+	val = ve_config_get_string (config, key);
 
 	if (strcmp (ve_sure_string (val), ve_sure_string (new_val)) != 0) {
-		gnome_config_set_string (key, new_val);
+		ve_config_set_string (config, key, new_val);
 
-		gnome_config_sync ();
+		ve_config_save (config, FALSE /* force */);
 
 		update_key (key);
 	}
 
 	g_free (val);
 
-	gnome_config_pop_prefix ();
-
 	return FALSE;
 }
 
@@ -316,12 +356,30 @@ timeout_remove (GtkWidget *widget)
 							"change_timeout"));
 	if (id != 0) {
 		g_source_remove (id);
+		g_object_set_data (G_OBJECT (widget), "change_timeout", NULL);
 	}
-	g_object_set_data (G_OBJECT (widget), "change_timeout", NULL);
 
 	func = g_object_get_data (G_OBJECT (widget), "timeout_func");
+	if (func != NULL) {
+		(*func) (widget);
+		g_object_set_data (G_OBJECT (widget), "timeout_func", NULL);
+	}
+}
 
-	(*func) (widget);
+static void
+timeout_remove_all (void)
+{
+	GList *li, *list;
+
+	list = timeout_widgets;
+	timeout_widgets = NULL;
+
+	for (li = list; li != NULL; li = li->next) {
+		g_print ("Removing %p!\n", li->data);
+		timeout_remove (li->data);
+		li->data = NULL;
+	}
+	g_list_free (list);
 }
 
 static void
@@ -333,9 +391,7 @@ setup_notify_toggle (const char *name,
 					      GTK_TYPE_TOGGLE_BUTTON);
 	gboolean val;
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-	val = gnome_config_get_bool (key);
-	gnome_config_pop_prefix ();
+	val = ve_config_get_bool (ve_config_get (GDM_CONFIG_FILE), key);
 
 	g_object_set_data_full (G_OBJECT (toggle),
 				"key", g_strdup (key),
@@ -348,8 +404,6 @@ setup_notify_toggle (const char *name,
 
 	g_signal_connect (G_OBJECT (toggle), "toggled",
 			  G_CALLBACK (toggle_toggled), NULL);
-	g_signal_connect (G_OBJECT (toggle), "destroy",
-			  G_CALLBACK (timeout_remove), NULL);
 }
 
 static void
@@ -361,9 +415,7 @@ setup_user_combo (const char *name, const char *key)
 	struct passwd *pwent;
 	char *str;
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-	str = gnome_config_get_string (key);
-	gnome_config_pop_prefix ();
+	str = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE), key);
 
 	/* normally empty */
 	users = g_list_append (users, g_strdup (""));
@@ -398,8 +450,6 @@ setup_user_combo (const char *name, const char *key)
 
 	g_signal_connect (G_OBJECT (entry), "changed",
 			  G_CALLBACK (entry_changed), NULL);
-	g_signal_connect (G_OBJECT (entry), "destroy",
-			  G_CALLBACK (timeout_remove), NULL);
 
 	g_list_foreach (users, (GFunc)g_free, NULL);
 	g_list_free (users);
@@ -415,9 +465,7 @@ setup_intspin (const char *name,
 					    GTK_TYPE_SPIN_BUTTON);
 	int val;
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-	val = gnome_config_get_int (key);
-	gnome_config_pop_prefix ();
+	val = ve_config_get_int (ve_config_get (GDM_CONFIG_FILE), key);
 
 	g_object_set_data_full (G_OBJECT (spin),
 				"key", g_strdup (key),
@@ -430,8 +478,6 @@ setup_intspin (const char *name,
 
 	g_signal_connect (G_OBJECT (spin), "value_changed",
 			  G_CALLBACK (intspin_changed), NULL);
-	g_signal_connect (G_OBJECT (spin), "destroy",
-			  G_CALLBACK (timeout_remove), NULL);
 }
 
 static gboolean
@@ -439,21 +485,19 @@ greeter_toggle_timeout (GtkWidget *toggle)
 {
 	const char *key = g_object_get_data (G_OBJECT (toggle), "key");
 	gboolean val;
+	VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-
-	val = gnome_config_get_bool (key);
+	val = ve_config_get_bool (config, key);
 
 	if ( ! ve_bool_equal (val, GTK_TOGGLE_BUTTON (toggle)->active)) {
-		gnome_config_set_bool (key, GTK_TOGGLE_BUTTON (toggle)->active);
+		ve_config_set_bool (config, key,
+				    GTK_TOGGLE_BUTTON (toggle)->active);
 
-		gnome_config_sync ();
+		ve_config_save (config, FALSE /* force */);
 
 		update_greeters ();
 	}
 
-	gnome_config_pop_prefix ();
-
 	return FALSE;
 }
 
@@ -471,9 +515,7 @@ setup_greeter_toggle (const char *name,
 					      GTK_TYPE_TOGGLE_BUTTON);
 	gboolean val;
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-	val = gnome_config_get_bool (key);
-	gnome_config_pop_prefix ();
+	val = ve_config_get_bool (ve_config_get (GDM_CONFIG_FILE), key);
 
 	g_object_set_data_full (G_OBJECT (toggle),
 				"key", g_strdup (key),
@@ -483,8 +525,6 @@ setup_greeter_toggle (const char *name,
 
 	g_signal_connect (G_OBJECT (toggle), "toggled",
 			  G_CALLBACK (greeter_toggle_toggled), NULL);
-	g_signal_connect (G_OBJECT (toggle), "destroy",
-			  G_CALLBACK (timeout_remove), NULL);
 }
 
 static gboolean
@@ -493,19 +533,18 @@ greeter_color_timeout (GtkWidget *picker)
 	const char *key = g_object_get_data (G_OBJECT (picker), "key");
 	char *val, *color;
 	guint8 r, g, b;
+	VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
 
 	gnome_color_picker_get_i8 (GNOME_COLOR_PICKER (picker),
 				   &r, &g, &b, NULL);
 	color = g_strdup_printf ("#%02x%02x%02x", (int)r, (int)g, (int)b);
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-
-	val = gnome_config_get_string (key);
+	val = ve_config_get_string (config, key);
 
 	if (strcmp (ve_sure_string (val), ve_sure_string (color)) != 0) {
-		gnome_config_set_string (key, ve_sure_string (color));
+		ve_config_set_string (config, key, ve_sure_string (color));
 
-		gnome_config_sync ();
+		ve_config_save (config, FALSE /* force */);
 
 		update_greeters ();
 	}
@@ -513,8 +552,6 @@ greeter_color_timeout (GtkWidget *picker)
 	g_free (val);
 	g_free (color);
 
-	gnome_config_pop_prefix ();
-
 	return FALSE;
 }
 
@@ -533,9 +570,7 @@ setup_greeter_color (const char *name,
 					      GNOME_TYPE_COLOR_PICKER);
 	char *val;
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-	val = gnome_config_get_string (key);
-	gnome_config_pop_prefix ();
+	val = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE), key);
 
 	g_object_set_data_full (G_OBJECT (picker),
 				"key", g_strdup (key),
@@ -551,8 +586,6 @@ setup_greeter_color (const char *name,
 
 	g_signal_connect (G_OBJECT (picker), "color_set",
 			  G_CALLBACK (greeter_color_changed), NULL);
-	g_signal_connect (G_OBJECT (picker), "destroy",
-			  G_CALLBACK (timeout_remove), NULL);
 
 	g_free (val);
 }
@@ -562,23 +595,20 @@ greeter_editable_timeout (GtkWidget *editable)
 {
 	const char *key = g_object_get_data (G_OBJECT (editable), "key");
 	char *text, *val;
+	VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
 
 	text = gtk_editable_get_chars (GTK_EDITABLE (editable), 0, -1);
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-
-	val = gnome_config_get_string (key);
+	val = ve_config_get_string (config, key);
 
 	if (strcmp (ve_sure_string (val), ve_sure_string (text)) != 0) {
-		gnome_config_set_string (key, ve_sure_string (text));
+		ve_config_set_string (config, key, ve_sure_string (text));
 
-		gnome_config_sync ();
+		ve_config_save (config, FALSE /* force */);
 
 		update_greeters ();
 	}
 
-	gnome_config_pop_prefix ();
-
 	g_free (text);
 	g_free (val);
 
@@ -599,9 +629,7 @@ setup_greeter_editable (const char *name,
 	char *val;
 	int pos;
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-	val = gnome_config_get_string (key);
-	gnome_config_pop_prefix ();
+	val = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE), key);
 
 	g_object_set_data_full (G_OBJECT (editable),
 				"key", g_strdup (key),
@@ -615,98 +643,28 @@ setup_greeter_editable (const char *name,
 
 	g_signal_connect (G_OBJECT (editable), "changed",
 			  G_CALLBACK (greeter_editable_changed), NULL);
-	g_signal_connect (G_OBJECT (editable), "destroy",
-			  G_CALLBACK (timeout_remove), NULL);
 
 	g_free (val);
 }
 
-static void
-whack_translations (const char *fullkey)
-{
-	char *section, *key, *p, *k, *v;
-	void *iterator;
-	GSList *to_clean, *li;
-
-	section = g_strdup (fullkey);
-	p = strchr (section, '/');
-	if (p == NULL) {
-		g_free (section);
-		return;
-	}
-	*p = '\0';
-	key = p+1;
-	p = strchr (key, '=');
-	if (p != NULL)
-		*p = '\0';
-
-
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-
-	to_clean = NULL;
-	iterator = gnome_config_init_iterator (section);
-	while ((iterator = gnome_config_iterator_next (iterator, &k, &v))
-	       != NULL) {
-		p = strchr (k, '[');
-		if (p != NULL) {
-			*p = '\0';
-			if (strcmp (key, k) == 0) {
-				*p = '[';
-				to_clean = g_slist_prepend
-					(to_clean,
-					 g_strconcat (section, "/", k, NULL));
-			}
-		}
-		g_free (k);
-		g_free (v);
-	}
-
-	if (to_clean != NULL) {
-		for (li = to_clean; li != NULL; li = li->next) {
-			char *key = li->data;
-			li->data = NULL;
-
-			gnome_config_clean_key (key);
-
-			g_free (key);
-		}
-
-		g_slist_free (to_clean);
-
-		gnome_config_sync ();
-	}
-
-	gnome_config_pop_prefix ();
-
-	g_free (section);
-}
-
 static gboolean
 greeter_entry_untranslate_timeout (GtkWidget *entry)
 {
 	const char *key = g_object_get_data (G_OBJECT (entry), "key");
 	const char *text;
-	char *val;
+	VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
+
+	g_print ("Timeout %p!\n", entry);
 
 	text = gtk_entry_get_text (GTK_ENTRY (entry));
 
-	whack_translations (key);
+	ve_config_delete_translations (config, key);
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
+	ve_config_set_string (config, key, ve_sure_string (text));
 
-	val = gnome_config_get_string (key);
+	ve_config_save (config, FALSE /* force */);
 
-	if (strcmp (ve_sure_string (val), ve_sure_string (text)) != 0) {
-		gnome_config_set_string (key, ve_sure_string (text));
-
-		gnome_config_sync ();
-
-		update_greeters ();
-	}
-
-	g_free (val);
-
-	gnome_config_pop_prefix ();
+	update_greeters ();
 
 	return FALSE;
 }
@@ -714,6 +672,7 @@ greeter_entry_untranslate_timeout (GtkWidget *entry)
 static void
 greeter_entry_untranslate_changed (GtkWidget *entry)
 {
+	g_print ("Changed %p!\n", entry);
 	run_timeout (entry, 500, greeter_entry_untranslate_timeout);
 }
 
@@ -724,9 +683,8 @@ setup_greeter_untranslate_entry (const char *name,
 	GtkWidget *entry = glade_helper_get (xml, name, GTK_TYPE_ENTRY);
 	char *val;
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-	val = gnome_config_get_translated_string (key);
-	gnome_config_pop_prefix ();
+	val = ve_config_get_translated_string (ve_config_get (GDM_CONFIG_FILE),
+					       key);
 
 	g_object_set_data_full (G_OBJECT (entry),
 				"key", g_strdup (key),
@@ -737,8 +695,6 @@ setup_greeter_untranslate_entry (const char *name,
 	g_signal_connect (G_OBJECT (entry), "changed",
 			  G_CALLBACK (greeter_entry_untranslate_changed),
 			  NULL);
-	g_signal_connect (G_OBJECT (entry), "destroy",
-			  G_CALLBACK (timeout_remove), NULL);
 
 	g_free (val);
 }
@@ -751,10 +707,9 @@ greeter_backselect_timeout (GtkWidget *toggle)
 					     GTK_TYPE_TOGGLE_BUTTON);
 	GtkWidget *image_bg = glade_helper_get (xml, "sg_image_bg_rb",
 						GTK_TYPE_TOGGLE_BUTTON);
+	VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-
-	val = gnome_config_get_int (GDM_KEY_BACKGROUNDTYPE);
+	val = ve_config_get_int (config, GDM_KEY_BACKGROUNDTYPE);
 
 	if (GTK_TOGGLE_BUTTON (no_bg)->active)
 		new_val = 0 /* No background */;
@@ -764,14 +719,12 @@ greeter_backselect_timeout (GtkWidget *toggle)
 		new_val = 2 /* Color */;
 
 	if (val != new_val) {
-		gnome_config_set_int (GDM_KEY_BACKGROUNDTYPE, new_val);
-		gnome_config_sync ();
+		ve_config_set_int (config, GDM_KEY_BACKGROUNDTYPE, new_val);
+		ve_config_save (config, FALSE /* force */);
 
 		update_greeters ();
 	}
 
-	gnome_config_pop_prefix ();
-
 	return FALSE;
 }
 
@@ -795,9 +748,8 @@ setup_greeter_backselect (void)
 	GtkWidget *color_bg = glade_helper_get (xml, "sg_color_bg_rb",
 						GTK_TYPE_TOGGLE_BUTTON);
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-	val = gnome_config_get_int (GDM_KEY_BACKGROUNDTYPE);
-	gnome_config_pop_prefix ();
+	val = ve_config_get_int (ve_config_get (GDM_CONFIG_FILE),
+				 GDM_KEY_BACKGROUNDTYPE);
 
 	if (val == 0)
 		gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (no_bg), TRUE);
@@ -812,10 +764,6 @@ setup_greeter_backselect (void)
 			  G_CALLBACK (greeter_backselect_toggled), NULL);
 	g_signal_connect (G_OBJECT (color_bg), "toggled",
 			  G_CALLBACK (greeter_backselect_toggled), NULL);
-
-	/* We set the timeout on the no_bg radiobutton */
-	g_signal_connect (G_OBJECT (no_bg), "destroy",
-			  G_CALLBACK (timeout_remove), NULL);
 }
 
 
@@ -845,9 +793,7 @@ setup_greeter_option (const char *name,
 	GtkWidget *option_menu = glade_helper_get (xml, name,
 						   GTK_TYPE_OPTION_MENU);
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-	val = gnome_config_get_string (key);
-	gnome_config_pop_prefix ();
+	val = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE), key);
 
 	if (val != NULL &&
 	    strcmp (val, EXPANDED_BINDIR "/gdmlogin --disable-sound --disable-crash-dialog") == 0) {
@@ -884,8 +830,6 @@ setup_greeter_option (const char *name,
 
 	g_signal_connect (G_OBJECT (option_menu), "changed",
 			  G_CALLBACK (option_changed), NULL);
-	g_signal_connect (G_OBJECT (option_menu), "destroy",
-			  G_CALLBACK (timeout_remove), NULL);
 }
 
 static void
@@ -1000,9 +944,8 @@ get_theme_dir (void)
 {
 	char *theme_dir;
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-	theme_dir = gnome_config_get_string (GDM_KEY_GRAPHICAL_THEME_DIR);
-	gnome_config_pop_prefix ();
+	theme_dir = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE),
+					  GDM_KEY_GRAPHICAL_THEME_DIR);
 
 	if (theme_dir == NULL ||
 	    theme_dir[0] == '\0' ||
@@ -1107,10 +1050,11 @@ read_themes (GtkListStore *store, const char *theme_dir, DIR *dir,
 	GtkTreeIter *select_iter = NULL;
 
 	while ((dent = readdir (dir)) != NULL) {
-		char *n, *key, *file, *name, *desc, *author, *copyright, *ss;
+		char *n, *file, *name, *desc, *author, *copyright, *ss;
 		char *full;
 		GtkTreeIter iter;
 		gboolean sel;
+		VeConfig *theme_file;
 		if (dent->d_name[0] == '.')
 			continue;
 		n = g_strconcat (theme_dir, "/", dent->d_name,
@@ -1125,11 +1069,10 @@ read_themes (GtkListStore *store, const char *theme_dir, DIR *dir,
 			continue;
 		}
 
-		key = g_strconcat ("=", n, "=/GdmGreeterTheme/", NULL);
-		gnome_config_push_prefix (key);
-		g_free (key);
+		theme_file = ve_config_new (n);
 
-		file = gnome_config_get_translated_string ("Greeter");
+		file = ve_config_get_translated_string
+			(theme_file, "GdmGreeterTheme/Greeter");
 		if (ve_string_empty (file)) {
 			g_free (file);
 			file = g_strconcat (dent->d_name, ".xml");
@@ -1141,7 +1084,6 @@ read_themes (GtkListStore *store, const char *theme_dir, DIR *dir,
 			g_free (file);
 			g_free (full);
 			g_free (n);
-			gnome_config_pop_prefix ();
 			continue;
 		}
 		g_free (full);
@@ -1151,16 +1093,22 @@ read_themes (GtkListStore *store, const char *theme_dir, DIR *dir,
 		else
 			sel = FALSE;
 
-		name = gnome_config_get_translated_string ("Name");
+		name = ve_config_get_translated_string
+			(theme_file, "GdmGreeterTheme/Name");
 		if (ve_string_empty (name)) {
 			g_free (name);
 			name = g_strdup (dent->d_name);
 		}
-		desc = gnome_config_get_translated_string ("Description");
-		author = gnome_config_get_translated_string ("Author");
-		copyright = gnome_config_get_translated_string ("Copyright");
-		ss = gnome_config_get_translated_string ("Screenshot");
-		gnome_config_pop_prefix ();
+		desc = ve_config_get_translated_string
+			(theme_file, "GdmGreeterTheme/Description");
+		author = ve_config_get_translated_string
+			(theme_file, "GdmGreeterTheme/Author");
+		copyright = ve_config_get_translated_string
+			(theme_file, "GdmGreeterTheme/Copyright");
+		ss = ve_config_get_translated_string
+			(theme_file, "GdmGreeterTheme/Screenshot");
+
+		ve_config_destroy (theme_file);
 
 		if (ss != NULL)
 			full = g_strconcat (theme_dir, "/", dent->d_name,
@@ -1204,19 +1152,17 @@ static gboolean
 greeter_theme_timeout (GtkWidget *toggle)
 {
 	char *val;
+	VeConfig *config = ve_config_get (GDM_CONFIG_FILE);
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-	val = gnome_config_get_string (GDM_KEY_GRAPHICAL_THEME);
+	val = ve_config_get_string (config, GDM_KEY_GRAPHICAL_THEME);
 
 	if (strcmp (ve_sure_string (val),
 		    ve_sure_string (selected_theme)) != 0) {
-		gnome_config_set_string (GDM_KEY_GRAPHICAL_THEME,
-					 selected_theme);
-		gnome_config_sync ();
+		ve_config_set_string (config, GDM_KEY_GRAPHICAL_THEME,
+				      selected_theme);
+		ve_config_save (config, FALSE /* force */);
 
 		update_greeters ();
-
-		gnome_config_pop_prefix ();
 	}
 
 	g_free (val);
@@ -1708,9 +1654,8 @@ setup_graphical_themes (void)
 
 	char *theme_dir = get_theme_dir ();
 
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-	selected_theme = gnome_config_get_string (GDM_KEY_GRAPHICAL_THEME);
-	gnome_config_pop_prefix ();
+	selected_theme = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE),
+					       GDM_KEY_GRAPHICAL_THEME);
 
 	/* create list store */
 	store = gtk_list_store_new (THEME_NUM_COLUMNS,
@@ -1785,18 +1730,23 @@ setup_graphical_themes (void)
 		gtk_tree_selection_select_iter (selection, select_iter);
 		g_free (select_iter);
 	}
+}
 
-	/* The change timeout is on the list */
-	g_signal_connect (G_OBJECT (theme_list), "destroy",
-			  G_CALLBACK (timeout_remove), NULL);
+static gboolean
+delete_event (GtkWidget *w)
+{
+	timeout_remove_all ();
+	gtk_main_quit ();
+	return FALSE;
 }
 
 static void
 dialog_response (GtkWidget *dlg, int response, gpointer data)
 {
 	if (response == GTK_RESPONSE_CLOSE) {
+		timeout_remove_all ();
 		gtk_main_quit ();
-	} else {
+	} else if (response == GTK_RESPONSE_HELP) {
 		/* FIXME: when help gets written connect it here */
 		GtkWidget *setup_dialog = glade_helper_get
 			(xml, "setup_dialog", GTK_TYPE_WINDOW);
@@ -1841,8 +1791,8 @@ setup_gui (void)
 				 GTK_TYPE_DIALOG,
 				 TRUE /* dump_on_destroy */);
 	dialog = glade_helper_get (xml, "setup_dialog", GTK_TYPE_DIALOG);
-	g_signal_connect (G_OBJECT (dialog), "destroy",
-			  G_CALLBACK (gtk_main_quit), NULL);
+	g_signal_connect (G_OBJECT (dialog), "delete_event",
+			  G_CALLBACK (delete_event), NULL);
 	g_signal_connect (G_OBJECT (dialog), "response",
 			  G_CALLBACK (dialog_response), NULL);
 
@@ -1944,6 +1894,88 @@ setup_gui (void)
 					 GDM_KEY_WELCOME);
 }
 
+static gboolean
+get_sensitivity (void)
+{
+	static Atom atom = 0;
+	Display *disp = gdk_x11_get_default_xdisplay ();
+	Window root = gdk_x11_get_default_root_xwindow ();
+	gulong *data;
+	gulong nitems_return;
+	gulong bytes_after_return;
+	Atom type_returned;
+	int format_returned;
+
+	if (atom == 0)
+		atom = XInternAtom (disp, "_GDM_SETUP_INSENSITIVE", False);
+
+	if (XGetWindowProperty (disp,
+				root,
+				atom,
+				0, 1,
+				False,
+				XA_CARDINAL,
+				&type_returned, &format_returned,
+				&nitems_return,
+				&bytes_after_return,
+				(unsigned char **)&data) != Success)
+		return TRUE;
+
+	if (format_returned != 32 ||
+	    data[0] == 0) {
+		XFree (data);
+		return TRUE;
+	} else {
+		XFree (data);
+		return FALSE;
+	}
+}
+
+static void
+update_sensitivity (void)
+{
+	gboolean sensitive = get_sensitivity ();
+	GtkWidget *setup_dialog = glade_helper_get (xml, "setup_dialog",
+						    GTK_TYPE_WINDOW);
+	gtk_widget_set_sensitive (setup_dialog, sensitive);
+	if (sensitive)
+		unsetup_window_cursor ();
+	else
+		setup_window_cursor (GDK_WATCH);
+}
+
+static GdkFilterReturn
+root_window_filter (GdkXEvent *gdk_xevent,
+		    GdkEvent *event,
+		    gpointer data)
+{
+	XEvent *xevent = (XEvent *)gdk_xevent;
+
+	if (xevent->type == PropertyNotify)
+		update_sensitivity ();
+
+	return GDK_FILTER_CONTINUE;
+}
+
+static void
+setup_disable_handler (void)
+{
+	XWindowAttributes attribs = { 0, };
+	Display *disp = gdk_x11_get_default_xdisplay ();
+	Window root = gdk_x11_get_default_root_xwindow ();
+
+	update_sensitivity ();
+
+	/* set event mask for events on root window */
+	XGetWindowAttributes (disp, root, &attribs);
+	XSelectInput (disp, root,
+		      attribs.your_event_mask |
+		      PropertyChangeMask);
+
+	gdk_window_add_filter (gdk_get_default_root_window (),
+			       root_window_filter, NULL);
+}
+
 static gboolean
 gdm_event (GSignalInvocationHint *ihint,
 	   guint		n_param_values,
@@ -2002,9 +2034,8 @@ main (int argc, char *argv[])
 		setup_cursor (GDK_WATCH);
 
 		/* If we are running under gdm parse the GDM gtkRC */
-		gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-		gtkrc = gnome_config_get_string (GDM_KEY_GTKRC);
-		gnome_config_pop_prefix ();
+		gtkrc = ve_config_get_string (ve_config_get (GDM_CONFIG_FILE),
+					      GDM_KEY_GTKRC);
 		if ( ! ve_string_empty (gtkrc))
 			gtk_rc_parse (gtkrc);
 		g_free (gtkrc);
@@ -2045,14 +2076,16 @@ main (int argc, char *argv[])
 
 	/* XXX: the setup proggie using a greeter config var for it's
 	 * ui?  Say it ain't so.  Our config sections are SUCH A MESS */
-	gnome_config_push_prefix ("=" GDM_CONFIG_FILE "=/");
-	GdmMinimalUID = gnome_config_get_int (GDM_KEY_MINIMALUID);
-	gnome_config_pop_prefix ();
+	GdmMinimalUID = ve_config_get_int (ve_config_get (GDM_CONFIG_FILE),
+					   GDM_KEY_MINIMALUID);
 
 	setup_gui ();
 
-	if (RUNNING_UNDER_GDM)
+	if (RUNNING_UNDER_GDM) {
+		setup_disable_handler ();
+
 		setup_cursor (GDK_LEFT_PTR);
+	}
 
 	gtk_main ();
 
